#!/usr/bin/env python
#
# vim:ts=8:sw=4
# file: profile_manager.py

import os
import sys

PROGRAM_NAME = 'booyo-config-wireless'

PROFILE_BASE_DIR = '/etc/sysconfig/networking/'
PROFILE_STORE_DIR = PROFILE_BASE_DIR + PROGRAM_NAME + '/profile/'
#PROFILE_STORE_DIR = './ifcfg_data'
IFCFG_SYSTEM_DIR = '/etc/sysconfig/network-scripts/'

class Profile:
	# 프로파일 이름을 셋팅한다.
	def set_profile_name(self, name=None):
		if not name:
			print 'profile_manager:17:argument is nothing'
			sys.exit(1)

		self.profile_attr_list[0] = name
	
	# 인터페이스 설정한다.
	def set_interface(self, interface=None):
		if not interface:
			print 'profile_manager:25:argument is nothing'
			sys.exit(1)

		self.profile_attr_list[1] = interface
	
	# ESSID를 설정한다.
	def set_essid(self, essid=None):
		if not essid:
			print 'profile_manager:33:argument is nothing'
			sys.exit(1)

		self.profile_attr_list[2] = essid
	
	# dhcp or static 설정한다.
	def set_dhcp_or_static(self, key=0, value=None):
		# value 타입이 list인지 검사한다.
		if key == 0 and value != None:
			print 'argument is failed'
			sys.exit(1)
		elif key == 1 and type(value) != type([]):
			print 'argument is failed'
			sys.exit(1)

		# key == 0, dhcp
		if key == 0:
			self.profile_attr_list[3] = 0
			self.profile_attr_list[4] = None
		elif key == 1:
			self.profile_attr_list[3] = 1
			self.profile_attr_list[4] = value
		else:
			print 'Ooops!!'
			sys.exit(1)

	# 프로파일 리스트를 가져간다.
	def get_profile_attr(self):
		if len(self.profile_attr_list) == 0:
			return None

		return self.profile_attr_list
	
	# 프로파일 이름을 리턴한다.
	def get_profile_name(self):
		return self.profile_attr_list[0]

	# ESSID의 이름을 리턴한다.
	def get_essid(self):
		return self.profile_attr_list[2]

	def __init__(self):
		print 'Profile constructor'
		self.profile_attr_list = ['', '', '', 0, None]


class ProfileManager:
	# 프로파일 디렉토리가 존재하는가?
	def is_exist_dir(self):
		dir_list = os.listdir(PROFILE_BASE_DIR)
		if len(dir_list) == 0:
			return 0
		
		for dir in dir_list:
			if dir == PROGRAM_NAME:
				return 1

		return 0
		
	# 프로파일을 저장하기 위한 저장소가 있는지 확인하고 없으면 만든다.
	def make_profile_storage(self):
		if self.is_exist_dir():
			print 'profile storage is already exist'
			return None
		else:
			print 'profile stotage is not exist'
			os.makedirs(PROFILE_STORE_DIR, 0644)
			return 1
	
	# 프로파일을 작성한다.
	# make_type == 0 : add
	# make_tpye == 1 : edit
	def make_profile(self, make_type=0, profile_name=None, interface=None, essid=None, key=0, value=None):
		if (not profile_name) or (not interface) or (not essid):
			print 'argument is failed'
			sys.exit(1)
		elif key == 0 and value != None:
			print 'argument is failed'
			sys.exit(1)
		elif key == 1 and type(value) != type([]):
			print 'argument is failed'
			sys.exit(1)

		profile = Profile()
		profile.set_profile_name(profile_name)
		profile.set_interface(interface)
		profile.set_essid(essid)
		profile.set_dhcp_or_static(key, value)
		
		if make_type == 0 and self.profile_list.has_key(profile_name):
			print 'Add: (%s) profile is already exist' % profile_name
			sys.exit(1)
		elif ( make_type == 1 ) and ( not self.profile_list.has_key(profile_name) ):
			print 'I think this problem more.'
			
			#print 'Edit: (%s) profile is not exist' % profile_name
			#sys.exit(1)

		self.profile_list[profile_name] = profile
	
	# profile을 삭제한다.
	def clear_profile(self, profile_name=None):
		if profile_name == None:
			print 'Not given profile name'
			sys.exit(1)

		if not self.profile_list.has_key(profile_name):
			print '(%s) profile is not exist'
			sys.exit(1)

		self.profile_list.pop(profile_name)

		# 실제 파일도 삭제처리 한다.
		path = PROFILE_STORE_DIR + profile_name
		os.remove(path)

	# 파일에 프로파일들을 작성한다.
	def commit_profile(self):
		print 'Profile file Writting...'
		if len(self.profile_list) == 0:
			print 'profile is empty'
			sys.exit(1)

		for name in self.profile_list.keys():
			if not name:
				print 'profile name is empty'
				continue
			path = PROFILE_STORE_DIR + name
			#path = PROFILE_STORE_DIR + 'ifcfg-' + name
			fw = open(path, 'w')

			fw.write('#This file is PROFILE information using ' + PROGRAM_NAME + '\n\n')

			# profile_name 저장
			fw.write(self.profile_title[0] + '=' + name + '\n')
			# profile 속성들 저장
			data = self.profile_list[name].get_profile_attr()
			if not data:
				print 'oops'
				sys.exit(1)
				
			fw.write(self.profile_title[1] + '=' + data[1] + '\n')
			fw.write(self.profile_title[2] + '=' + data[2] + '\n')
			if data[3] == 0:
				fw.write(self.profile_title[3] + '=' + 'dhcp' + '\n')
			elif data[3] == 1:
				fw.write(self.profile_title[4] + '=' + 'static' + '\n')
				value = data[4]
				if ( not value ) or ( type(value) != type([]) ):
					print 'oops'
					sys.exit(1)
				fw.write(self.profile_title[5] + '=' + value[0] + '\n')
				fw.write(self.profile_title[6] + '=' + value[1] + '\n')
				fw.write(self.profile_title[7] + '=' + value[2] + '\n')
			else:
				print 'oops'
				sys.exit(1)
			fw.close()

	# 프로파일들을 읽는다.
	def read_profile(self):
		print 'Profile Reading...'
		if not self.is_exist_dir():
			print 'profile is not exist'
			return None
		
		try:
			file_list = os.listdir(PROFILE_STORE_DIR)
		except OSError, msg:
			print 'user is NOT ROOT!: %s' % msg
			return None

		if len(file_list) == 0:
			print 'profile is not exist'
			return None
		
		for file in file_list:
			path = PROFILE_STORE_DIR + file
			fo = open(path, 'r')

			tmp_profile = Profile()
			key = None
			value = ['','','']
			for line in fo.readlines():
				if line[0] == '#':
					#print 'This statement is comment %s' % line
					continue
				if line.split('=')[0] == self.profile_title[0]:
					tmp_profile.set_profile_name(line.split('=')[1].split('\n')[0])
				elif line.split('=')[0] == self.profile_title[1]:
					tmp_profile.set_interface(line.split('=')[1].split('\n')[0])
				elif line.split('=')[0] == self.profile_title[2]:
					tmp_profile.set_essid(line.split('=')[1].split('\n')[0])
				elif line.split('=')[0] == self.profile_title[3]:
					key = 0
				elif line.split('=')[0] == self.profile_title[4]:
					key = 1
				elif line.split('=')[0] == self.profile_title[5]:
					value[0] = line.split('=')[1].split('\n')[0]
				elif line.split('=')[0] == self.profile_title[6]:
					value[1] = line.split('=')[1].split('\n')[0]
				elif line.split('=')[0] == self.profile_title[7]:
					value[2] = line.split('=')[1].split('\n')[0]
				else:
					print 'Unknown attribute : %s' % line
					continue

			if key == 0:
				tmp_profile.set_dhcp_or_static(0)
			elif key == 1:
				tmp_profile.set_dhcp_or_static(1, value)
				
			self.profile_list[tmp_profile.get_profile_name()] = tmp_profile

			fo.close()

	def get_profile_data(self, profile_name=None):
		if not profile_name:
			print 'profile_manager:247:argument is nothing'
			sys.exit(1)

		if not self.profile_list.has_key(profile_name):
			return None

		return self.profile_list[profile_name]

	# OLD APIs
	# 프로파일 이름들만 리턴해준다.
	def get_profile_names(self):
		if len(self.profile_list.keys()) == 0:
			return None
		return self.profile_list.keys()

	# 프로파일들의 ESSID값만 리턴해준다.
	def get_essid_names(self):
		essid_name = []
		for profile in self.profile_list.keys():
			essid_name.append(self.profile_list[profile].get_essid())

		if len(essid_name) == 0:
			return None
		return essid_name

	def set_profile_data(self, data=None):
		if ( not data ) or ( type(data) != type({}) ):
			print 'profile_manger:260:argument is nothing'
			sys.exit(1)
		title = ['profile', 'interface', 'essid', 'radio', 'ip', 'network', 'netmask']
		radio_value = ['dhcp', 'static']

		# dhcp
		if data[title[3]] == radio_value[0]:
			self.make_profile(1, data[title[0]], data[title[1]], data[title[2]], 0)
		elif data[title[3]] == radio_value[1]:
			value = [data[title[4]], data[title[5]], data[title[6]]]
			self.make_profile(1, data[title[0]], data[title[1]], data[title[2]], 1, value)

	def write(self, data=None):
		self.commit_profile()
		

	def __init__(self):
		print 'Profile Manager contructor'
		self.profile_list = {}
		self.profile_title = ['PROFILE_NAME', 'INTERFACE', 'ESSID', 'DHCP', 'STATIC', 'IPADDRESS', 'GATEWAY', 'NETMASK']

		if not self.make_profile_storage():
			self.read_profile()

	def __del__(self):
		print 'Profile Manager destructor...'
		self.commit_profile()


if __name__ == "__main__":
    ProfileManager()
