#!/usr/bin/env python
# vim:ts=4:sw=2

import sys
import pygtk
pygtk.require('2.0')
import gtk
import gobject

sys.path.append('../backend')
import dlg_ap
import dlg_error
import ap_scan
import ifcfg_manager
#import net_device_manager
import dlg_access

class BooyoAP:

	def __init__ (self, parent_widget=None):
		self.parent = parent_widget

	def init_window (self):
		self.show_tabs = True
		self.show_border = True

		self.parent = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.parent.connect("destroy", lambda w: gtk.main_quit())
		self.parent.set_title("AP Searching module")
		self.parent.set_size_request(500, 500)
	
		layout = self.init_gui()	
		self.parent.add(layout)

		self.parent.show()
		
	def init_gui(self):

		main_vbox = gtk.VBox(False, 1)
		main_vbox.set_border_width(1)

		frame = gtk.Frame()
		frame.set_border_width(10)
		frame.set_size_request(400, 310)
		frame.show()
		main_vbox.pack_start(frame, False, True, 0)
		
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		frame.add(scrolled_window)
		scrolled_window.show()

		self.model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
		self.tree_view = gtk.TreeView(self.model)
		scrolled_window.add_with_viewport(self.tree_view)
		self.tree_view.show()

		cell1 = gtk.CellRendererText()
		cell2 = gtk.CellRendererText()
		cell3 = gtk.CellRendererText()

		column1 = gtk.TreeViewColumn( _('ESSID'), cell1, text=0 )
		column1.set_min_width(100)
		column1.set_alignment(0.5)
		column2 = gtk.TreeViewColumn( _('Speed'), cell2, text=1 )
		column2.set_min_width(100)
		column2.set_alignment(0.5)
		column3 = gtk.TreeViewColumn( _('Address'), cell3, text=2 )
		column3.set_min_width(100)
		column3.set_alignment(0.5)

		self.tree_view.append_column(column1)
		self.tree_view.append_column(column2)
		self.tree_view.append_column(column3)

		button_hbox = gtk.HBox(False, 1)
		button_hbox.set_border_width(1)
		button_hbox.show()

		apply_button = gtk.Button("Apply", gtk.STOCK_APPLY)
		apply_button.set_size_request(100,30)
		apply_button.connect("clicked", self.callback_apply, "apply")
		apply_button.set_sensitive(False)
		apply_button.show()
		
		refresh_button = gtk.Button("Refresh", gtk.STOCK_REFRESH)
		refresh_button.set_size_request(100,30)
		refresh_button.connect("clicked", self.callback_refresh, "refresh", apply_button)
		refresh_button.show()

		button_hbox.pack_end(apply_button, False, True, 0);
		button_hbox.pack_end(refresh_button, False, True, 0);

		main_vbox.pack_start(button_hbox, False, True, 0)
		main_vbox.show()

		self.tree_view.connect("cursor-changed", self.callback_view_changed, "view_changed", apply_button)
		self.refresh_func()
	
		return main_vbox

	def return_dialog_data(self, data=None):
		if not data:
			print 'argument is nothing'
			sys.exit(1)
		if type(data) != type({}):
			print 'data type is not same'
			sys.exit(1)

		# ifcfg_convertor에 연결한다.
		convertor = ifcfg_manager.Convertor()
		ifcfgmanager, device = convertor.convert_ap(data)
		ifcfgmanager.write_ifcfg()

		# 네트워크를 재시작한다.
		access = dlg_access.DlgAccess()
		access.run(device)

######################
# Callback Functions #
######################
	def callback_view_changed(self, widget, data, apply_button):
		print 'treeview cursor changed'
		treemodel, iter = self.tree_view.get_selection().get_selected()
		if treemodel.get_value(iter, 0):
			apply_button.set_sensitive(True)
	
	def callback_apply(self, widget, data):
		print 'apply button clicked'
		if self.parent:
			self.dialog = dlg_ap.DlgAP(self)	
			treemodel, iter = self.tree_view.get_selection().get_selected()
			if iter == None:
				print 'iter exception'
			else:
				self.dialog.set_essid(treemodel.get_value(iter, 0))
				self.dialog.set_title( _('Acess Pointer Connect') )
		else:
			print 'exception'
		
	def callback_refresh(self, widget, data, apply_button):
		self.refresh_func()
		apply_button.set_sensitive(False)
			
	def refresh_func(self):
		print 'refresh button clicked'
		str = ap_scan.read_iwlist()
		if len(str) <= 1:
			print 'exception'
		
		APList = ap_scan.parser(str)
		self.model.clear()	

		if APList:
			for ap in APList:
				entry = APList[ap]
				self.model.insert(0, [entry['essid'], '%s Mb/s' % entry['bitrate'], entry['address']])			

def main():
    gtk.main()
    return 0

if __name__ == "__main__":
	booyoap = BooyoAP()
	booyoap.init_window()
	main()
