#!/usr/bin/env python
#
# vim:ts=4:sw=2
# file: gui_profile.py

import pygtk
pygtk.require('2.0')
import os
import gtk
import sys
import gobject

import dlg_profile
import dlg_error

import profile_manager
import ifcfg_manager
import dlg_access

class GuiProfile:
	def col_list_1th(self):
		list = self.manager.get_profile_names()
		if not list:
			return []
		return list

	def col_list_2th(self):
		list = self.manager.get_essid_names()
		if not list:
			return []
		return list

	def col_list(self, model=None):
		if not model:
			print 'argument is nothing'
			sys.exit(1)

		name_list = self.col_list_1th()
		essid_list = self.col_list_2th()
		
		for name in name_list:
			iter = model.append()
			model.set(iter, 0, name, 1, essid_list.pop(0))

	def create_list(self):
		# Create a new scrolled window, with scrollbars only if needed
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

		model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
		self.tree_view = gtk.TreeView(model)
		scrolled_window.add_with_viewport (self.tree_view)
		self.tree_view.show()

		cell1 = gtk.CellRendererText()
		cell2 = gtk.CellRendererText()

		column1 = gtk.TreeViewColumn( _('Profile Name'), cell1, text=0 )
		column1.set_min_width(100)
		column1.set_alignment(0.5)

		column2 = gtk.TreeViewColumn( _('ESSID'), cell2, text=1 )
		column2.set_min_width(100)
		column2.set_alignment(0.5)

		self.tree_view.append_column(column1)
		self.tree_view.append_column(column2)

		self.tree_view.connect("button-press-event", self.callback_button_press)

		self.col_list(model)

		return scrolled_window

	def profile_tab_create(self, window, notebook=None):
		if notebook == None:
			print 'argument is nothing'
			sys.exit(1)

		self.window = window
		vbox = gtk.VBox(False, 1)
		vbox.set_border_width(1)
		notebook.add(vbox)
		vbox.show()

		frame = gtk.Frame()
		frame.set_border_width(10)
		frame.set_size_request(400, 310)
		frame.show()

		scrolled_window = self.create_list()
		frame.add(scrolled_window)
		scrolled_window.show()

		hbox_tmp = gtk.HBox()
		hbox_tmp.show()

		image = gtk.Image()
		image.set_from_stock(gtk.STOCK_ABOUT, gtk.ICON_SIZE_MENU)
		hbox_tmp.pack_start(image, False, False, 0)
		image.show()

		label = gtk.Label( ' ' + _('Profiles') )
		hbox_tmp.pack_start(label, False, False, 0)
		label.show()

		notebook.set_tab_label(vbox, hbox_tmp)

		vbox.pack_start(frame, False, True, 0)

		button_hbox = gtk.HBox(False, 1)
		button_hbox.set_border_width(1)
		button_hbox.show()
		vbox.pack_start(button_hbox, False, True, 0)

		button = gtk.Button("CONNECT", gtk.STOCK_APPLY)
		button.set_size_request(100,30)
		button.set_sensitive(False)
		button.connect("clicked", self.callback_apply_button, "test")
		button.show()
		button_hbox.pack_end(button, False, True, 0);

		button_add = gtk.Button("Add", gtk.STOCK_ADD)
		button_add.set_size_request(100,30)
		button_add.connect("clicked", self.callback_profile_add_button, "test")
		button_add.show()
		button_hbox.pack_start(button_add, False, True, 0);

		button_sub = gtk.Button("Sub", gtk.STOCK_REMOVE)
		button_sub.set_size_request(100,30)
		button_sub.connect("clicked", self.callback_profile_sub_button, "test")
		button_sub.show()
		button_hbox.pack_start(button_sub, False, True, 0);

		button_mod = gtk.Button("Mod", gtk.STOCK_EDIT)
		button_mod.set_size_request(100,30)
		button_mod.connect("clicked", self.callback_profile_mod_button, "test")
		button_mod.show()
		button_hbox.pack_start(button_mod, False, True, 0);

		self.tree_view.connect("cursor-changed", self.callback_view_selected, button)

	def return_dialog_data(self, data=None):
		if not data:
			print 'argument is nothing'
			sys.exit(1)
		if type(data) != type({}):
			print 'data type is not same'
			sys.exit(1)

		self.manager.set_profile_data(data)
		self.manager.write()

	def clear_model(self):
		model = self.tree_view.get_model()
		model.clear()

	def add_model(self):
		model = self.tree_view.get_model()
		if not model:
			print 'not get model'

		self.col_list(model)

#####################
# Callback Function #
#####################
	def callback_view_selected(self, widget, data=None):
		if widget.get_selection():
			data.set_sensitive(True)
		else:
			data.set_sensitive(False)

	def callback_button_press(self, widget, event, data=None):
		if event.type == gtk.gdk._2BUTTON_PRESS:
			treemodel, iter = self.tree_view.get_selection().get_selected()
			if not iter:
				print 'iter is nothing'
				error = dlg_error.DlgError( _('Select profile!') )
				return None
			
			dialog = dlg_profile.DlgProfile(self)

			name_col, essid_col = treemodel.get(iter, 0, 1)

			# 존재하는 프로파일 정보를 가져와서 뿌려준다.
			profile_attr = self.manager.get_profile_data(name_col).get_profile_attr()
			
			# profile object를 주면 다이얼로그 박스에서 알아서 설정하는 부분 필요
			dialog.set_dialog_data(profile_attr)
		else:
			print 'not 2 button press event'
			pass	
			
	def callback_profile_add_button(self, widget, data=None):
		print 'profile add button clicked'
		dlg = dlg_profile.DlgProfile(self)
		if not dlg:
			print 'not create DlgProfile'
			sys.exit(1)

		dlg.set_title( _('Add Profile') )
		
		self.clear_model()
		self.add_model()

	def callback_profile_mod_button(self, widget, data=None):
		print 'profile modify button clicked'
		treemodel, iter = self.tree_view.get_selection().get_selected()
		if not iter:
			print 'iter is empty'
			error = dlg_error.DlgError( _('Select profile!') )
			return None

		dialog = dlg_profile.DlgProfile(self)
		dialog.set_title( _('Profile Edition') )

		name_col, essid_col = treemodel.get(iter, 0, 1)

		# 존재하는 프로파일 정보를 가져와서 뿌려준다.
		profile_attr = self.manager.get_profile_data(name_col).get_profile_attr()
		
		# profile object를 주면 다이얼로그 박스에서 알아서 설정하는 부분 필요
		dialog.set_dialog_data(profile_attr)
		
	def callback_profile_sub_button(self, widget, data=None):
		print 'profile remove button clicked'
		model, iter = self.tree_view.get_selection().get_selected()
		if not iter:
			print 'iter is empty'
			error = dlg_error.DlgError( _('Select delete profile!') )
			return None

		name_col, essid_col = model.get(iter, 0, 1)
		self.manager.clear_profile(name_col)

		self.clear_model()
		self.col_list(model)

	def callback_apply_button(self, widget, data=None):
		print 'apply button clicked'
		model, iter = self.tree_view.get_selection().get_selected()
		if not iter:
			print 'iter is empty'
			error = dlg_error.DlgError( _('Select delete profile!') )
			return None

		name_col, essid_col = model.get(iter, 0, 1)

		# profile의 dictionary구조를 넘겨주면 ifcfg_manager에서 변경해주는 작업
		ifcfg_convertor = ifcfg_manager.Convertor()
		ifcfgmanager, device = ifcfg_convertor.convert_profile(self.manager.get_profile_data(name_col).get_profile_attr())
		ifcfgmanager.write_ifcfg()

		access = dlg_access.DlgAccess()
		access.run(device)

	def __init__(self):
		print 'GUI Profile Constructor...'

		self.manager = profile_manager.ProfileManager()
